unit DevSet;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Clipbrd, XComErr;

type
  TFDev = class(TForm)
    GB2               : TGroupBox;
    GB1               : TGroupBox;
    BOK               : TButton;
    BCancel           : TButton;
    L1                : TLabel;
    L2                : TLabel;
    L3                : TLabel;
    L4                : TLabel;
    L5                : TLabel;
    L6                : TLabel;
    eName             : TEdit;
    eBaud             : TEdit;
    cbFlow            : TComboBox;
    cbParity          : TComboBox;
    cbData            : TComboBox;
    cbStop            : TComboBox;
    procedure BaudKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure BaudKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ApplySettings;
    procedure CancelSettings;
  end;

var
  FDev: TFDev;

implementation
uses XComDrv, Main;

{$R *.DFM}

procedure TFDev.BaudKeyPress(Sender: TObject; var Key: Char);
begin
  if not (Key in [#8, #13, '0'..'9']) then Key := #0;
end;

procedure TFDev.FormCreate(Sender: TObject);
var i: Char;
begin
  for i:=#0 to #3 do cbFlow.Items.Add(STR_FLOWCONTROL[TFlowControl(i)]);
  for i:='4' to '8' do cbData.Items.Add(i);
  for i:=#0 to #4 do cbParity.Items.Add(STR_PARITY[TParity(i)]);
  with cbStop, Items do
  begin
    Add('1');
    Add('1.5');
    Add('2');
  end;
  CancelSettings;
end;

procedure TFDev.ApplySettings;
begin
  with FMain.Modem, DataControl do
  begin
    BaudValue := StrToInt(eBaud.Text);
    FlowControl := TFlowControl(cbFlow.ItemIndex);
    DataBits := TDataBits(cbData.ItemIndex);
    Parity := TParity(cbParity.ItemIndex);
    StopBits := TStopBits(cbStop.ItemIndex);
  end;
end;

procedure TFDev.CancelSettings;
begin
  with FMain.Modem, DataControl do
  begin
    eBaud.Text := IntToStr(BaudValue);
    cbFlow.ItemIndex := Integer(FlowControl);
    cbData.ItemIndex := Integer(DataBits);
    cbParity.ItemIndex := Integer(Parity);
    cbStop.ItemIndex := Integer(StopBits);
    eName.Text := DeviceName;
  end;
end;

procedure TFDev.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if ModalResult = mrOK then
  with FMain, Modem do
  begin
    if Opened then BeginUpdate;
    ApplySettings;
    if not Opened then DeviceName := eName.Text
    else
    try
      if EndUpdate then SaveProfile
      else raise EDeviceError.Create(SInvalidDCB);
    except
      on E: EDeviceError do
      begin
        LoadProfile;
        CancelSettings;
        CanClose := False;
        MessageDlg(E.Message, mtError, [mbOK], 0);
      end;
    end;
  end else CancelSettings;
end;

procedure TFDev.FormShow(Sender: TObject);
begin
  eName.Enabled := not FMain.Modem.Opened;
end;

procedure TFDev.BaudKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  s: string;
  i: Integer;
begin
  if (Shift = [ssCtrl]) and (Key=86) then
  begin
    Key := 0;
    s := Clipboard.AsText;
    Caption := s;
    if (s<>'') then
    begin
      i := 1;
      while (i<=Length(s)) do
        if (s[i] in ['0'..'9'])
          then Inc(i)
          else Delete(s, i, 1);
      eBaud.SelText := s;
    end;
  end;
end;

end.
