unit Main;
{$H+,B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs,
  Menus, XComDrv, ToolWin, ComCtrls, ExtCtrls, Buttons,
  StdCtrls, Graphics;

type

  TXMemo = class(TMemo)
  protected
    procedure WndProc(var Message: TMessage); override;
  end;

  TFMain = class(TForm)
    Menu               : TMainMenu;
    Modem              : TXModem;
    CB                 : TControlBar;
    P1                 : TPanel;
    P2                 : TPanel;
    P3                 : TPanel;
    S1                 : TBevel;
    S2                 : TBevel;
    S3                 : TBevel;
    Bevel1             : TBevel;    
    Edit               : TEdit;
    DSend              : TButton;
    DOpen              : TSpeedButton;
    DClose             : TSpeedButton;
    DSet               : TSpeedButton;
    MOpt               : TSpeedButton;
    MInit              : TSpeedButton;
    MReset             : TSpeedButton;
    MCon               : TSpeedButton;
    MDis               : TSpeedButton;
    DCState            : TSpeedButton;
    SBar               : TStatusBar;    
    N5           : TMenuItem;
    ECS1         : TMenuItem;
    Device1      : TMenuItem;
    Open1        : TMenuItem;
    Close1       : TMenuItem;
    N1           : TMenuItem;
    Settings1    : TMenuItem;
    N2           : TMenuItem;
    Exit1        : TMenuItem;
    Modem1       : TMenuItem;
    Options1     : TMenuItem;
    N3           : TMenuItem;
    Initialize1  : TMenuItem;
    Reset1       : TMenuItem;
    N4           : TMenuItem;
    Connect1     : TMenuItem;
    Disconnect1  : TMenuItem;
    procedure DCloseClick(Sender: TObject);
    procedure DOpenClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure DSetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DSendClick(Sender: TObject);
    procedure MInitClick(Sender: TObject);
    procedure MResetClick(Sender: TObject);
    procedure ModemHayesAT(Sender: TObject; AT: THayesAT);
    procedure MOptClick(Sender: TObject);
    procedure MConClick(Sender: TObject);
    procedure MDisClick(Sender: TObject);
    procedure ModemDisconnect(Sender: TObject);
    procedure ModemRing(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ModemConnecting(Sender: TObject;
      const ConnectType: TConnectType);
    procedure ModemDisconnecting(Sender: TObject);
    procedure ModemConnect(Sender: TObject);
    procedure Modem1Click(Sender: TObject);
    procedure DCStateClick(Sender: TObject);
    procedure ECS1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ModemRead(Sender: TObject);
    procedure ModemSend(Sender: TObject);
    procedure ModemChangeCmdState(Sender: TObject);
    procedure ModemCommEvent(Sender: TObject; const Events: TDeviceEvents);
    procedure ModemData(Sender: TObject; const Received: Cardinal);
  private
    { Private declarations }
    Memo: TXMemo;
    InConnecting: Boolean;
  public
    { Public declarations }
    procedure SaveProfile;
    procedure LoadProfile;
  end;

var
  FMain: TFMain;

implementation

uses DevSet, ModSet;

{$R *.DFM}

procedure TXMemo.WndProc(var Message: TMessage);
begin
  inherited WndProc(Message);
  if (Message.Msg = WM_SETFOCUS) then HideCaret(Handle);
end;

type
  TProfile = record
    Baud    : DWORD;
    Flow    : TFlowControl;
    DB      : TDataBits;
    Par     : TParity;
    SB      : TStopBits;
  end;

var
  Profile: TProfile;
  BufStr: string='';

procedure TFMain.SaveProfile;
begin
  with Modem, DataControl, Profile do
  begin
    Baud := BaudValue;
    Flow := FlowControl;
    DB := DataBits;
    Par := Parity;
    SB := StopBits;
  end;
end;

procedure TFMain.LoadProfile;
begin
  with Modem, DataControl, Profile do
  begin
    BaudValue := Baud;
    FlowControl := Flow;
    DataBits := DB;
    Parity := Par;
    StopBits := SB;
  end;
end;

procedure TFMain.DCloseClick(Sender: TObject);
begin
  if not Modem.Opened then Exit;
  if (msConnected in Modem.ModemState) then
  begin
    Memo.Lines.Add('Closing while connected.');
    Modem.LowerDTR;
  end;
  Modem.CloseDevice;
  SBar.Panels[0].Text := 'Idle...';
  SBar.Panels[1].Text := '';
  SBar.Panels[2].Text := '';
  Close1.Enabled := False;
  DClose.Enabled := False;
  Open1.Enabled := True;
  DOpen.Enabled := True;
  MOpt.Enabled := True;
  MInit.Enabled := False;
  MReset.Enabled := False;
  MCon.Enabled := False;
  MDis.Enabled := False;
  Edit.Enabled := False;
  DSend.Enabled := False;
  Options1.Enabled := DOpen.Enabled;
  Initialize1.Enabled := MInit.Enabled;
  Reset1.Enabled := MReset.Enabled;
  Connect1.Enabled := MCon.Enabled;
  Disconnect1.Enabled := MDis.Enabled;
  DCState.Enabled := False;
  DCState.Hint := '';
  Memo.Lines.Add('Device closed.');
end;

procedure TFMain.DOpenClick(Sender: TObject);
begin
  if Modem.Opened then Exit;
  with Memo, Lines do
  begin
    Clear;
    Repaint;
    Add('Opening device.');
    Add('Profile:');
    Add('  Port >          '+FDev.eName.Text);
    Add('  Baud rate >     '+FDev.eBaud.Text);
    Add('  Flow control >  '+FDev.cbFlow.Text);
    Add('  Data bits >     '+FDev.cbData.Text);
    Add('  Parity >        '+FDev.cbParity.Text);
    Add('  Stop bits >     '+FDev.cbStop.Text);
  end;

  SBar.Panels[1].Text := 'Received: 0';
  SBar.Panels[2].Text := 'Sent: 0';
  try
    Close1.Enabled := Modem.OpenDevice;
    if Modem.Opened
      then Memo.Lines.Add('Device succesfully opened.')
      else Memo.Lines.Add('Failed to open device.');
  except
    on E: Exception do
      Memo.Lines.Add('Failed to open device. '+E.Message);
  end;

  if Modem.Opened
    then SBar.Panels[0].Text := 'Device opened'
    else Exit;

  DClose.Enabled := True;
  Open1.Enabled := False;
  DOpen.Enabled := False;

  Edit.Enabled := True;
  DSend.Enabled := True;
  DCState.Enabled := Modem.Installed;
  DCState.Hint := 'Enter command state';

end;

procedure TFMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TFMain.DSetClick(Sender: TObject);
begin
  FDev.ShowModal;
end;

procedure TFMain.FormCreate(Sender: TObject);
begin
  InConnecting := False;
  Modem.BaudRate := brCustom;
  Memo := TXMemo.Create(Self);
  Memo.Parent := Self;
  Memo.WordWrap := False;
  Memo.ScrollBars := ssBoth;
  Memo.Align := alClient;
  Memo.ReadOnly := True;
  Memo.TabStop := False;
  Memo.Cursor := crArrow;
  with Memo.Font do
  begin
    Size := 8;
    Style := [];
    Name := 'Courier New';
    Color := clBlack;
  end;
  SaveProfile;
  SBar.Canvas.Font := SBar.Font;
end;

procedure TFMain.DSendClick(Sender: TObject);
var Async: THandle;
begin
  if Edit.Enabled and not Edit.Focused then Edit.SetFocus;
  if (msCommandState in Modem.ModemState) then
    Modem.SendCommand(Edit.Text)
  else if not Modem.Locked then
  begin
    Modem.PurgeIn;
    BufStr := '';
    Async := Modem.InitAsync(nil, True);
    Modem.SendStringAsync(Async, Edit.Text+#13);
  end;
end;

procedure TFMain.MInitClick(Sender: TObject);
begin
  with Memo.Lines, Modem, ModemSettings do
  begin
    Add('Initializing modem.');
    Add(InitString);
    if InitModem
      then Add('Passed.')
      else Add('Failed.');
  end;
end;

procedure TFMain.MResetClick(Sender: TObject);
begin
  with Memo.Lines, Modem, ModemSettings do
  begin
    Add('Resetting modem.');
    Add(ResetString);
    if ResetModem
      then Add('Passed.')
      else Add('Failed.');
  end;
end;

procedure TFMain.ModemHayesAT(Sender: TObject; AT: THayesAT);
begin
  if InConnecting then
  begin
    InConnecting := False;
    if (AT.ME<>ME_CONNECT) then Exit;
  end;
  Memo.Lines.Add('Received command:');
  Memo.Lines.Add(AT.Command);
  if (AT.Data<>'') then Memo.Lines.Add(AT.Data);
  Memo.Lines.Add(AT.Result);
end;

procedure TFMain.MOptClick(Sender: TObject);
begin
  FMod.ShowModal;
end;

procedure TFMain.MConClick(Sender: TObject);
var s: string;
begin
  case Modem.ModemSettings.ConnectType of
    ctDial:
    begin
      s:='ATD';
      case Modem.ModemSettings.DialType of
        dtTone: s := s+'T';
        dtPulse: s := s+'P';
      end;
      s:=s+' '+Modem.ModemSettings.DialNumber;
      Modem.SendCommand(s);
    end;
    ctDirect:
    begin
      s := 'ATA';
      Modem.SendCommand(s);
    end;
    ctWait: Modem.SetRegisterValue(0, Modem.ModemSettings.WaitRings);
  end;
end;

procedure TFMain.MDisClick(Sender: TObject);
begin
  SBar.Panels[0].Text := 'Disconnecting...';
  if Modem.Connected and (coLeaveDTROpen in Modem.Options)
    then Modem.LowerDTR
    else Modem.Disconnect;
  if not Modem.Opened then Exit;
  SBar.Panels[0].Text := 'Device opened';
  MOpt.Enabled := True;
  MInit.Enabled := True;
  MReset.Enabled := True;
  MCon.Enabled := True;
  MDis.Enabled := False;
  Initialize1.Enabled := MInit.Enabled;
  Reset1.Enabled := MReset.Enabled;
  Connect1.Enabled := MCon.Enabled;
  Options1.Enabled := MOpt.Enabled;
  Disconnect1.Enabled := MDis.Enabled;
end;

procedure TFMain.ModemDisconnect(Sender: TObject);
begin
  Memo.Lines.Add('Disconnected');
  SBar.Panels[0].Text := 'Device opened';
end;

procedure TFMain.ModemRing(Sender: TObject);
begin
  Memo.Lines.Add('RING detected.');
end;

procedure TFMain.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then DSend.Click;
end;

procedure TFMain.ModemConnecting(Sender: TObject;
  const ConnectType: TConnectType);
begin
  InConnecting := True;
  with Memo.Lines do
  begin
    Add('Connecting.');
    case ConnectType of
      ctDial:   SBar.Panels[0].Text := 'Dialing...';
      ctWait:   SBar.Panels[0].Text := 'Waiting for call...';
      ctDirect: SBar.Panels[0].Text := 'Establishing direct connection...';
    end;

    Add(SBar.Panels[0].Text);
    MInit.Enabled := False;
    MReset.Enabled := False;
    MCon.Enabled := False;
    MDis.Enabled := True;
    DCState.Enabled := False;
    DCState.Hint := '';
    Initialize1.Enabled := MInit.Enabled;
    Reset1.Enabled := MReset.Enabled;
    Connect1.Enabled := MCon.Enabled;
    Disconnect1.Enabled := True;
  end;
end;

procedure TFMain.ModemDisconnecting(Sender: TObject);
begin
  if InConnecting then
    Memo.Lines.Add('Unable to connect.');
  SBar.Panels[0].Text := 'Device opened';
  MOpt.Enabled := True;
  MInit.Enabled := True;
  MReset.Enabled := True;
  MCon.Enabled := True;
  MDis.Enabled := False;
  Initialize1.Enabled := MInit.Enabled;
  Reset1.Enabled := MReset.Enabled;
  Connect1.Enabled := MCon.Enabled;
  Options1.Enabled := MOpt.Enabled;
  Disconnect1.Enabled := MDis.Enabled;
  DCState.Hint := 'Exit command state';
  DCState.Down := True;
  DCState.Enabled := True;
end;

procedure TFMain.ModemConnect(Sender: TObject);
begin
  InConnecting := False;
  DCState.Enabled := True;
  DCState.Down := False;
  DCState.Hint := 'Enter command state';
  Memo.Lines.Add('Connected');
end;

procedure TFMain.Modem1Click(Sender: TObject);
begin
  if Modem.Opened and Modem.Installed then
  begin
    ECS1.Visible := True;
    if msCommandState in Modem.ModemState
      then ECS1.Caption := '&Exit command state'
      else ECS1.Caption := '&Enter command state';
  end else ECS1.Visible := False;
  N5.Visible := ECS1.Visible;
end;

procedure TFMain.DCStateClick(Sender: TObject);
var Update: Boolean;
begin
  if (Modem.BaudValue>=MB_MINBAUD) then
  begin
    if (msCommandState in Modem.ModemState)
      then Update := not Modem.ExitCommandState
      else Update := not Modem.EnterCommandState;
  end else Update := True;
  if Update then
    DCState.Down := not DCState.Down;
end;

procedure TFMain.ECS1Click(Sender: TObject);
begin
  DCState.Click;
  DCState.Down := not DCState.Down;
end;

procedure TFMain.FormDestroy(Sender: TObject);
begin
  Memo.Free;
end;

const MW = 80;

procedure TFMain.ModemRead(Sender: TObject);
var
  s: string;
  I: Integer;
begin
  s := Format('Received: %d', [Modem.TotalReceived]);
  I := SBar.Canvas.TextWidth(s)+6;
  if I<MW then I := MW;
  SBar.Panels[1].Width := I;
  SBar.Panels[1].Text := s;
end;

procedure TFMain.ModemSend(Sender: TObject);
var
  s: string;
  I: Integer;
begin
  s := Format('Sent: %d', [Modem.TotalSent]);
  I := SBar.Canvas.TextWidth(s)+6;
  if I<MW then I := MW;
  SBar.Panels[2].Width := I;
  SBar.Panels[2].Text := s;
end;

procedure TFMain.ModemChangeCmdState(Sender: TObject);
begin
  if (msCommandState in Modem.ModemState) then
  begin
    DCState.Hint := 'Exit command state';
    Memo.Lines.Add('Entered into command state');
    DCState.Down := True;
  end else
  begin
    DCState.Hint := 'Enter command state';
    Memo.Lines.Add('Command state not present');
    DCState.Down := False;    
  end;
  if (msCommandState in Modem.ModemState) then
  begin
    MInit.Enabled := not Modem.Connected;
    MReset.Enabled := not Modem.Connected;
    MCon.Enabled := not Modem.Connected;
    MDis.Enabled := Modem.Connected;
  end else
  begin
    MInit.Enabled := False;
    MReset.Enabled := False;
    MCon.Enabled := False;
    MDis.Enabled := False;
  end;
  Initialize1.Enabled := MInit.Enabled;
  Reset1.Enabled := MReset.Enabled;
  Connect1.Enabled := MCon.Enabled;
  Disconnect1.Enabled := MDis.Enabled;
end;

type tcs= set of char;

procedure RemoveChars( const ch: tcs; var s: string );
var i: integer;
begin
  i := 1;
  while (i<=Length(s)) do
    if s[i] in ch then
      Delete(s, i, 1)
    else
      Inc(i);
end;

function PBreak(var s: string): Integer;
begin
  for Result:=1 to Length(s) do
    if s[Result]<#32 then Exit;
  Result := 0;
end;

function GetLine(var s, s1: string): Boolean;
var I: Integer;
begin
  I := PBreak(s);
  if (I>0) then
  begin
    s1 := Copy(s, 1, I-1);
    Delete(s, 1, I);
    Result := True;
    RemoveChars([#0..#31], s1);
  end else Result := False;
end;

procedure TFMain.ModemCommEvent(Sender: TObject;
  const Events: TDeviceEvents);
begin
  DCState.Enabled := Modem.Installed;
end;

procedure TFMain.ModemData(Sender: TObject; const Received: Cardinal);
var
  s, s1: string;
begin
  if not Modem.Locked and Modem.ReadString(s) then
  begin
    if not Modem.Connected then
    begin
      s := BufStr+s;
      while GetLine(s, s1) do
      if (s1<>'') then Memo.Lines.Add(s1);
      BufStr := s;
    end else
      Memo.Lines.Add(s);
  end;
end;

end.
