unit XComErr;
(**************************************************************************
 *     This unit is part of 2labs Communication components.               *
 *     Copyright (C) 2001 Alexander Grischenko and Ondrej Urik.           *
 **************************************************************************)

{$DEFINE EXC} //Enabling exceptions

interface

uses
  Windows, SysUtils;

resourcestring
  SInvalidBaudRate = '''%s'' is not a valid baud rate value.';
  SInvalidFlowControl = '''%s'' is not a valid flow control value.';
  SInvalidDataBits = '''%s'' is not a valid data bits value.';
  SInvalidParity = '''%s'' is not a valid parity value.';
  SInvalidStopBits = '''%s'' is not a valid stop bits value.';

  SDOpenError  = 'Cannot open device %s';
  SCommOpened = 'Device is allready opened.';
  SCommClosed = 'Device not opened.';
  SInvalidDCB = 'Unable to update device control block.';
  SInvalidIOSize = 'Unable to update IO buffers.';
  SLockedSend = 'Device locked. Cannot perform send operations.';
  SLockedRead = 'Device locked. Cannot perform read operations.';
  SNoCommandState = 'Modem command state not present.';
  SEnumFailed = 'EnumPorts failed.'; // raise XCommWin32Error
  SReadError = 'Unable to read from port.'; // raise XCommWin32Error
  SSendError = 'Unable to write to port.'; //raise XCommWin32Error

const
  {Error codes}
  DEC_OPENERROR            = $FFFF;
  DEC_CLOSED               = $0001;
  DEC_OPENED               = $0002;
  DEC_INVALIDDCB           = $0003;
  DEC_LOCKEDSEND           = $0004;
  DEC_LOCKEDREAD           = $0005;
  DEC_INVALIDIOSIZE        = $0006;
  DEC_NOCOMMANDSTATE       = $0007;
  DEC_NOTCONNECTED         = $0008;
  DEC_READERROR            = $000A;
  DEC_SENDERROR            = $000B;
  DEC_ENUMFAILED           = $000C;

type
  EDeviceError = class ( Exception )
  public
    ErrorCode: integer;
    constructor CreateCode( const Msg: string; const Code: integer ); virtual;
  end;

procedure XCommError( const Ident: string; const Code: Integer );
procedure XCommWin32Error( const Ident: string; const Code: Integer );

implementation

constructor EDeviceError.CreateCode( const Msg: string; const Code: Integer );
begin
  Create(Msg);
  ErrorCode := Code;
end;

procedure XCommError( const Ident: string; const Code: Integer );
begin
{$IFDEF EXC}
  raise EDeviceError.CreateCode(Ident, Code);
{$ENDIF}
end;

procedure XCommWin32Error( const Ident: string; const Code: Integer );
var ErrCode32: DWORD;
begin
{$IFDEF EXC}
  ErrCode32 := GetLastError;
  raise EDeviceError.CreateCode(
     Format('%s (Win32 error %d: $s)',
     [Ident,ErrCode32,SysErrorMessage(ErrCode32)]),
  Code);
{$ENDIF}
end;

end.
